
###############################################################################################
#  AN EXAMPLE TO FIT LOGISTIC UNIDIMENSIONAL ITEM RESPONSE MODELS: Classical Inference                       #
#                                                                                             #
# Data contain 974 respondent to 18 items of Math Test applied in student of  4th grade.      #
# Data are described in Bolfarine, H. and Bazn, J. L. (2010).  Bayesian Estimation of the    #   
# Logistic Positive Exponent IRT model. Journal of Educational Behavioral Statistics, 35,     #
# 693-713.                                                                                    #
###############################################################################################



# install package ltm, psych, foreign, psychometric, irtoys
#install.packages(c("ltm","psych","foreign", "psychometric","irtoys") 

#read data set


library(ltm)
require(foreign)

wd<-"C:\\Users\\Jorge Luis\\Copy\\MinicursTRI\\Exemplos\\Unidimensional"
setwd(wd)

mate=read.spss("Mathpear.sav")
datamat=data.frame(mate)



#Classical item analysis 
require(ltm)
cronbach.alpha(datamat)

require(psych)
alpha(datamat)


# Evaluation of multidimensionality

fa(datamat) #factor analysis using pearson correlation matrix

require(psych)
c=tetrachoric(datamat)

fap1 <- fa(r=c$rho)  #factor analysis using tetrachoric correlation matrix
fap1
fap2 <- fa(r=c$rho,nfactors=2,rotate="varimax")
fap2

par(mfrow=c(1,2))
fa.diagram(fap2,cut=0.4)
factor.plot(fap2, cut=0.4,ylim=c(0,1),xlim=c(0,1))
abline(h=0.4,col=4,lty=2)
abline(v=0.4,col=4,lty=2)



#ML Estimation of Logistic IRT unidimensional models using ltm: Calibration of items
#with usual parameterization, models 1L, 2L and 3L
#################################################################################

#Using ltm
require(ltm)
#Estimation One Logistic IRT Model 1L
fit1L=rasch(datamat,constraint=cbind(length(datamat)+1,1))
summary(fit1L)

#Estimation Two Logistic IRT Model 2L
fit2L=ltm(datamat~z1,IRT.param=TRUE)
summary(fit2L)

#Estimation Three Logistic IRT Model 2L
fit3L=tpm(datamat,type="latent.trait",IRT.param=TRUE,max.guessing=1)
summary(fit3L)

#Using irtoys
require(irtoys)
#Estimation One Logistic IRT Model 1L
fit1La<- est(datamat, model="1PL", engine="ltm")
fit1La

fit1La$est[,2]

#Estimation Two Logistic IRT Model 2L
fit2La<- est(datamat, model="2PL", engine="ltm")
fit2La

#Estimation Two Logistic IRT Model 2L
fit3La<- est(datamat, model="3PL", engine="ltm")
fit3La

#Comparison of estimation of difficults parameters under differents IRT models and Classical difficulty
require(psychometric)

#classical difficulty (proportion of correct response)
score=apply(datamat,1,sum)
itemanalisis<-item.exam(datamat, y=score, discrim=TRUE)
p<-itemanalisis$Difficulty

difficulties=matrix(c(
1-p,
log((1-p)/p),
fit1La$est[,2],
fit2La$est[,2],
fit3La$est[,2]),
nrow=ncol(datamat))
colnames(difficulties) <- c("prop","prop tran","b 1L","b 2L","b 3L")
difficulties

difficultiesest=500+10*difficulties[,-1]
difficultiesest

cor(difficultiesest)
par(mfrow=c(1,1))
matplot(difficultiesest)
#plot.matrix(difficultiesest)

#Estimation of abilities
detach("package:psych", unload=TRUE)

#2L using ltm
EAP=factor.scores(fit2L, method = "EAP")$score.dat
write.table(EAP,"C:/Users/Jorge Luis/Copy/MiniCursTRI/Exemplos/Unidimensional/EAPmathpear.csv",sep=",") 


#2L using irtoys considering different methods

#Maximum Likelihood estimation
th.mle2L <- mlebme(resp=datamat, ip=fit2La$est, method="ML")
th.mle2L

#Expected a Posteriori
th.eap2L <- eap(resp=datamat, ip=fit2La$est, qu=normal.qu())
th.eap2L 

#Mode a Posteriori
th.map2L <- mlebme(resp=datamat, ip=fit2La$est, method="BM")
th.map2L

#Weight Likelihood estimation
th.wle2L <- wle(resp=datamat, ip=fit2La$est)
th.wle2L



thetas=matrix(c(
score,
scale(score),
th.mle2L[,1],
th.eap2L[,1],
th.map2L[,1],
th.wle2L[,1]),
nrow=nrow(datamat))
colnames(thetas) <- c("Score","Z score","MLE","EAP","MAP","WLE")
thetas

thetast=500+10*thetas[,-1]
thetast
cor(thetast)
matplot(thetast)

plot(thetast[,1],thetast[,3],xlim=c(470,525),ylim=c(470,525),xlab="Z score",ylab="EAP")
lines(thetast[,1],thetast[,1])

#3L Model

#Maximum Likelihood estimation
th.mle3L <- mlebme(resp=datamat, ip=fit3La$est, method="ML")
th.mle3L

#Expected a Posteriori
th.eap3L <- eap(resp=datamat, ip=fit3La$est, qu=normal.qu())
th.eap3L 

#Mode a Posteriori
th.map3L <- mlebme(resp=datamat, ip=fit3La$est, method="BM")
th.map3L

#Weight Likelihood estimation
th.wle3L <- wle(resp=datamat, ip=fit3La$est)
th.wle3L



thetas3L=matrix(c(
score,
scale(score),
th.mle3L[,1],
th.eap3L[,1],
th.map3L[,1],
th.wle3L[,1]),
nrow=131)
colnames(thetas3L) <- c("Score","Z Score","MLE","EAP","MAP","WLE")
thetas3L


thetast3L=500+10*thetas3L[,-1]
thetast3L
cor(thetast3L)
matplot(thetast3L)

plot(thetast[,1],thetast[,3],xlim=c(470,525),ylim=c(470,525),xlab="Z score",ylab="EAP")
lines(thetast[,1],thetast[,1])



# Note. Reviewed your results with carefully  Compare your results with the obtained 
# using other software. For example you can use IRTPRO
# http://www.ssicentral.com/irt/student.html


